<?php
/**
 * @copyright 2010-2015 JTL-Software GmbH
 * @package jtl\Connector\Model
 * @subpackage Product
 */

namespace jtl\Connector\Model;

use DateTime;
use JMS\Serializer\Annotation as Serializer;

/**
 *
 * @access public
 * @package jtl\Connector\Model
 * @subpackage Product
 * 
 * @Serializer\AccessType("public_method")
 */
class ProductStockLevel extends DataModel
{
    /**
     * @var Identity 
     * @Serializer\Type("jtl\Connector\Model\Identity")
     * @Serializer\SerializedName("productId")
     * @Serializer\Accessor(getter="getProductId",setter="setProductId")
     */
    protected $productId = null;

    /**
     * @var string Optional stock keeping unit identifier
     * @Serializer\Type("string")
     * @Serializer\SerializedName("sku")
     * @Serializer\Accessor(getter="getSku",setter="setSku")
     */
    protected $sku = '';

    /**
     * @var double 
     * @Serializer\Type("double")
     * @Serializer\SerializedName("stockLevel")
     * @Serializer\Accessor(getter="getStockLevel",setter="setStockLevel")
     */
    protected $stockLevel = 0.0;

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->productId = new Identity();
    }

    /**
     * @param Identity $productId 
     * @return ProductStockLevel
     * @throws \InvalidArgumentException if the provided argument is not of type 'Identity'.
     */
    public function setProductId(Identity $productId)
    {
        return $this->setProperty('productId', $productId, 'Identity');
    }

    /**
     * @return Identity 
     */
    public function getProductId()
    {
        return $this->productId;
    }

    /**
     * @return string
     */
    public function getSku(): string
    {
        return $this->sku;
    }

    /**
     * @param string $sku
     * @return ProductStockLevel
     */
    public function setSku(string $sku): ProductStockLevel
    {
        $this->sku = $sku;
        return $this;
    }

    /**
     * @param double $stockLevel 
     * @return ProductStockLevel
     */
    public function setStockLevel($stockLevel)
    {
        return $this->setProperty('stockLevel', $stockLevel, 'double');
    }

    /**
     * @return double 
     */
    public function getStockLevel()
    {
        return $this->stockLevel;
    }

    /**
     * @param array|string[] $publics
     * @return stdClass|\stdClass
     */
    public function getPublic(array $publics = array('fields', 'isEncrypted', 'identities', '_type'))
    {
        $publics[] = 'sku';
        return parent::getPublic($publics); // TODO: Change the autogenerated stub
    }
}
